==> improv-0.0.0.tgz: improv-0.0.0/improv_signal.c <==

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <improv/improv.h>


ImprovSignal *improv_signal_new(void)
{
	ImprovSignal *sig;

	sig=(ImprovSignal *)malloc(sizeof(ImprovSignal));
	memset(sig,0,sizeof(ImprovSignal));

	return sig;
}

void improv_signal_free(ImprovSignal *sig)
{
	free(sig);
}

int improv_signal_connect(ImprovObject *obj,int signal_type,ImprovSignalFunc
	func,void *arg)
{
	ImprovSignal *sig;

	sig=improv_signal_new();

	sig->next=(ImprovSignal *)obj->signals;
	sig->object=obj;
	sig->signal_type=signal_type;
	sig->func=func;
	sig->arg=arg;

	obj->signals=sig;

	return 0;
}

static ImprovSignal *pending,*pending_last;

int improv_signal_emit(ImprovObject *object,int signal_type)
{
	ImprovSignal *sig;

	sig=improv_signal_new();

	sig->object=object;
	sig->signal_type=signal_type;
	
	if(pending_last){
		pending_last->next=sig;
	}
	pending_last=sig;
	if(!pending)
		pending=sig;

	return 0;
}

int improv_signal_marshal(ImprovObject *obj,int signal_type)
{
	ImprovSignal *sig;
	int did=0;

	sig=(ImprovSignal *)obj->signals;
	while(sig){
printf("signal type=%d\n",sig->signal_type);
		if(sig->signal_type==signal_type)
			sig->func(obj,signal_type,sig->arg);
		sig=sig->next;
		did++;
	}
	if(!did){
		printf("no signals of type %d\n",signal_type);
	}
	return did;
}

int improv_signal_marshal_one(void)
{
	ImprovSignal *sig=pending;

	if(!pending){
		printf("no signals to marshal\n");
		return 0;
	}

	pending=sig->next;
	if(!pending)pending_last=pending;

	improv_signal_marshal(sig->object,sig->signal_type);

	improv_signal_free(sig);

	if(pending)return 1;
	return 0;
}

==> improv-0.0.1.tgz: improv-0.0.1/improv_signal.c <==

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <improv/improv.h>


ImprovSignal *improv_signal_new(void)
{
	ImprovSignal *sig;

	sig=(ImprovSignal *)malloc(sizeof(ImprovSignal));
	memset(sig,0,sizeof(ImprovSignal));

	printf("new signal <%p>\n",sig);

	return sig;
}

void improv_signal_free(ImprovSignal *sig)
{
	free(sig);
}

int improv_signal_connect(ImprovObject *obj,int signal_type,ImprovSignalFunc
	func,void *arg)
{
	ImprovSignal *sig;

	sig=improv_signal_new();

	sig->next=(ImprovSignal *)obj->signals;
	sig->object=obj;
	sig->signal_type=signal_type;
	sig->func=func;
	sig->arg=arg;

	obj->signals=sig;

	return 0;
}

static ImprovSignal *pending,*pending_last;

int improv_signal_emit(ImprovObject *object,int signal_type)
{
	ImprovSignal *sig;

	sig=improv_signal_new();

	sig->object=object;
	sig->signal_type=signal_type;
	
	if(pending_last){
		pending_last->next=sig;
	}
	pending_last=sig;
	if(!pending)
		pending=sig;

printf("emit signal type=%d from <%p> to <%p>\n",sig->signal_type,NULL,object);

	return 0;
}

int improv_signal_marshal(ImprovObject *obj,int signal_type)
{
	ImprovSignal *sig;
	int did=0;

	sig=IMPROV_SIGNAL(obj->signals);
	while(sig){
		if(sig->signal_type==signal_type){
			printf("marshal signal <%p> type=%d from <%p> to <%p>\n",sig,sig->signal_type,NULL,obj);
			sig->func(obj,signal_type,sig->arg);
		}
		sig=sig->next;
		did++;
	}
	if(!did){
		printf("no signals of type %d\n",signal_type);
	}
	return did;
}

int improv_signal_marshal_one(void)
{
	ImprovSignal *sig=pending;

	if(!pending){
		printf("no signals to marshal\n");
		return 0;
	}

	pending=sig->next;
	if(!pending)pending_last=pending;

	improv_signal_marshal(sig->object,sig->signal_type);

	improv_signal_free(sig);

	if(pending)return 1;
	return 0;
}

int improv_signal_forward(ImprovObject *obj,int signal_type,void *arg)
{
	improv_signal_emit(IMPROV_OBJECT(arg),signal_type);

	return 0;
}
